@@ -20,6 +20,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
+#ident "$Id: genksyms.c,v 1.2 1997/09/11 00:07:17 rth Exp $"
 
 #include <stdio.h>
 #include <string.h>
@@ -49,6 +50,7 @@ static int errors;
 static int nsyms;
 
 static struct symbol *expansion_trail;
+static const char *crc_prefix = "";
 
 static const char * const symbol_type_name[] = {
   "normal", "typedef", "enum", "struct", "union"
@@ -407,11 +409,15 @@ export_symbol(const char *name)
 
       if (checksum_version > 1)
 	{
-	  fprintf(outfile, "#define __ver_%s\t%08lx\n", name, crc);
+	  fprintf(outfile, "#define __ver_%s\t%s%08lx\n", name,
+		  crc_prefix, crc);
 	  fprintf(outfile, "#define %s\t_set_ver(%s)\n", name, name);
 	}
       else
-	fprintf(outfile, "#define %s\t_set_ver(%s, %08lx)\n", name, name, crc);
+	{
+	  fprintf(outfile, "#define %s\t_set_ver(%s, %s%08lx)\n", name, name,
+		  crc_prefix, crc);
+	}
     }
 }
 
@@ -458,7 +464,7 @@ main(int argc, char **argv)
 {
   int o;
 
-  while ((o = getopt(argc, argv, "dgwqVDk:")) != EOF)
+  while ((o = getopt(argc, argv, "dgwqVDk:p:")) != EOF)
     switch (o)
       {
       case 'd':
@@ -498,18 +504,22 @@ main(int argc, char **argv)
 	  kernel_version = a << 16 | b << 8 | c;
 	}
         break;
+      case 'p':
+	crc_prefix = optarg;
+	break;
 
       default:
       usage:
 	fputs("Usage:\n"
 	      "genksyms [-dDwqV] -k kernel_version > .../linux/module/*.ver\n"
 	      "\n"
-	      "  -d       Debug level -- repeat to increase\n"
-	      "  -D       Dump final exported definitions\n"
-	      "  -w       Enable warnings (aka errors)\n"
-	      "  -q       Disable warnings (default)\n"
-	      "  -V       Show version\n"
-	      "  -k ver   Set the kernel version for which we are compiling\n"
+	      "  -d        Debug level -- repeat to increase\n"
+	      "  -D        Dump final exported definitions\n"
+	      "  -w        Enable warnings (aka errors)\n"
+	      "  -q        Disable warnings (default)\n"
+	      "  -V        Show version\n"
+	      "  -k ver    Set the kernel version for which we are compiling\n"
+	      "  -p string Set a mangling prefix for all symbols\n"
 	      , stderr);
 	return 1;
       }
