@@ -19,7 +19,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_common.c,v 1.1 1997/09/10 22:20:02 rth Exp $"
+#ident "$Id: obj_common.c,v 1.3 1998/01/23 09:33:38 rth Exp $"
 
 #include <string.h>
 #include <assert.h>
@@ -87,11 +87,13 @@ obj_set_symbol_compare (struct obj_file *f,
 }
 
 struct obj_symbol *
-obj_add_symbol (struct obj_file *f, const char *name,
+obj_add_symbol (struct obj_file *f, const char *name, unsigned long symidx,
 		int info, int secidx, ElfW(Addr) value, unsigned long size)
 {
   struct obj_symbol *sym;
   unsigned long hash = f->symbol_hash(name) % HASH_BUCKETS;
+  int n_type = ELFW(ST_TYPE)(info);
+  int n_binding = ELFW(ST_BIND)(info);
 
   for (sym = f->symtab[hash]; sym; sym = sym->next)
     if (f->symbol_cmp(sym->name, name) == 0)
@@ -100,8 +102,6 @@ obj_add_symbol (struct obj_file *f, const char *name,
 	int o_info = sym->info;
 	int o_type = ELFW(ST_TYPE)(o_info);
 	int o_binding = ELFW(ST_BIND)(o_info);
-	int n_type = ELFW(ST_TYPE)(info);
-	int n_binding = ELFW(ST_BIND)(info);
 
 	/* A redefinition!  Is it legal?  */
 
@@ -109,10 +109,36 @@ obj_add_symbol (struct obj_file *f, const char *name,
 	  return sym;
 	else if (o_secidx == SHN_UNDEF)
 	  goto found;
+	else if (n_binding == STB_GLOBAL && o_binding == STB_LOCAL)
+	  {
+	    /* Cope with local and global symbols of the same name
+	       in the same object file, as might have been created
+	       by ld -r.  The only reason locals are now seen at this
+	       level at all is so that we can do semi-sensible things
+	       with parameters.  */
+	    
+	    struct obj_symbol *nsym, **p;
+
+	    nsym = arch_new_symbol();
+  	    nsym->next = sym->next;
+	    nsym->ksymidx = -1;
+
+	    /* Excise the old (local) symbol from the hash chain.  */
+	    for (p = &f->symtab[hash]; *p != sym; p = &(*p)->next)
+	      continue;
+	    *p = sym = nsym;
+	    goto found;
+	  }
 	else if (n_binding == STB_LOCAL)
-	  goto found;
-	else if (o_binding == STB_LOCAL)
-	  return sym;
+	  {
+	    /* Another symbol of the same name has already been defined.
+	       Just add this to the local table.  */
+	    sym = arch_new_symbol();
+	    sym->next = NULL;
+	    sym->ksymidx = -1;
+	    f->local_symtab[symidx] = sym;
+	    goto found;
+	  }
 	else if (n_binding == STB_WEAK)
 	  return sym;
 	else if (o_binding == STB_WEAK)
@@ -141,6 +167,9 @@ obj_add_symbol (struct obj_file *f, const char *name,
   f->symtab[hash] = sym;
   sym->ksymidx = -1;
 
+  if (ELFW(ST_BIND)(info) == STB_LOCAL)
+    f->local_symtab[symidx] = sym;
+
 found:
   sym->name = name;
   sym->value = value;
@@ -194,34 +223,30 @@ obj_find_section (struct obj_file *f, const char *name)
 }
 
 static int
-obj_load_order_cmp(struct obj_section *a, struct obj_section *b)
+obj_load_order_prio(struct obj_section *a)
 {
-  unsigned long af, bf, ac, bc;
+  unsigned long af, ac;
 
   af = a->header.sh_flags;
-  bf = b->header.sh_flags;
 
   ac = 0;
+  if (a->name[0] != '.' || strlen(a->name) != 10 ||
+      strcmp(a->name + 5, ".init")) ac |= 32;
   if (af & SHF_ALLOC) ac |= 16;
   if (!(af & SHF_WRITE)) ac |= 8;
   if (af & SHF_EXECINSTR) ac |= 4;
   if (a->header.sh_type != SHT_NOBITS) ac |= 2;
 
-  bc = 0;
-  if (bf & SHF_ALLOC) bc |= 16;
-  if (!(bf & SHF_WRITE)) bc |= 8;
-  if (bf & SHF_EXECINSTR) bc |= 4;
-  if (b->header.sh_type != SHT_NOBITS) bc |= 2;
-
-  return bc - ac;
+  return ac;
 }
 
 void
 obj_insert_section_load_order (struct obj_file *f, struct obj_section *sec)
 {
   struct obj_section **p;
+  int prio = obj_load_order_prio(sec);
   for (p = f->load_order_search_start; *p ; p = &(*p)->load_next)
-    if (obj_load_order_cmp(sec, *p) < 0)
+    if (obj_load_order_prio(*p) < prio)
       break;
   sec->load_next = *p;
   *p = sec;
